TITAN_REAGENTTRACKER_ID = "ReagentTracker"

local L = LibStub("AceLocale-3.0"):GetLocale("Titan", true)
local _, class = UnitClass("player")

local _, addon = ...


-- store spells that are known here later
local possessed = {{}, {}, {}, {}}


function TitanPanelReagentTrackerButton_OnLoad(self)
	self.registry = {
		id = TITAN_REAGENTTRACKER_ID,
		version = GetAddOnMetadata("TitanReagentTracker", "Version"),
		menuText = "Reagent Tracker",
		tooltipTitle = "Reagent Tracker Info", 
		tooltipTextFunction = "TitanPanelReagentTrackerButton_GetTooltipText",
		savedVariables = {
			TrackReagent1 = true,
			TrackReagent2 = false,
			TrackReagent3 = false,
			TrackReagent4 = false,
			ShowSpellIcons = false,
		}
	}
	
	-- don't continue if no buffs are associated with the class
	if addon.buffs[class] then
		self:RegisterEvent("PLAYER_LOGIN")
		self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	end
end


local queryTooltip = CreateFrame("GameTooltip", "TitanReagentTrackerTooltip", nil, "GameTooltipTemplate")
queryTooltip:SetOwner(UIParent, "ANCHOR_NONE")
queryTooltip:SetScript("OnTooltipSetItem", function(self)
	if TitanReagentTrackerTooltipTextLeft1:GetText() ~= RETRIEVING_ITEM_INFO then
		TitanPanelReagentTracker_RefreshReagents()
		TitanPanelReagentTracker_UpdateButton()
		TitanPanelButton_UpdateTooltip(TitanPanelReagentTrackerButton)
	end
end)


function TitanPanelReagentTrackerButton_OnEvent(self, event, ...)
	if event == "PLAYER_LOGIN" then
		TitanPanelReagentTracker_RefreshReagents()
		TitanPanelReagentTracker_UpdateButton()
		TitanPanelButton_UpdateTooltip(self)
		self:RegisterEvent("BAG_UPDATE")
	else
		-- update on next frame to prevent redundant CPU processing from event spamming
		self.refreshReagents = event == "LEARNED_SPELL_IN_TAB"
		self:SetScript("OnUpdate", TitanPanelReagentTrackerButton_OnUpdate)
		return
	end
end


function TitanPanelReagentTrackerButton_OnUpdate(self, elapsed)
	if self.refreshReagents then
		TitanPanelReagentTracker_RefreshReagents()
		self.refreshReagents = false
	end
	TitanPanelReagentTracker_UpdateButton()
	TitanPanelButton_UpdateTooltip(self)
	self:SetScript("OnUpdate", nil)
end


function TitanPanelReagentTracker_RefreshReagents()
	for index, buff in ipairs(addon.buffs[class]) do
		local possessed = possessed[index]
		wipe(possessed)
		for i = 1, #buff.spellIDs do
			local name, rank, icon = GetSpellInfo((GetSpellInfo(buff.spellIDs[i])))
			if name then
				local rank = tonumber(rank:match("(%d+)"))
				-- use the first reagent in the table if reagentIDs[rank] doesn't exist (multi spell reagent)
				local reagentID = buff.reagentIDs[rank] or buff.reagentIDs[1]
				local reagentName = GetItemInfo(reagentID)
				if not reagentName then
					queryTooltip:SetHyperlink("item:"..reagentID)
					return
				end
				local reagentIcon = GetItemIcon(reagentID)
				-- add this spell to our "known spells" table
				-- if this reagent is already tracked through another spell, we'll want to hide it
				possessed.disabled = TitanPanelReagentTracker_IsReagentTracked(reagentName)
				possessed.reagentName = reagentName
				possessed.reagentIcon = reagentIcon
				possessed.spellIcon = icon
				break
			end
		end
	end
end


function TitanPanelReagentTracker_IsReagentTracked(reagent)
	for i, v in pairs(possessed) do
		-- ignore already ignored reagents
		if v.reagentName == reagent and not v.disabled then
			return true
		end
	end
end


function TitanPanelReagentTracker_UpdateButton()
	local tracking
	local totalWidth = 0
	local offset = 0
	local name = "TitanPanelReagentTrackerButtonReagent"
	
	for index, buff in ipairs(possessed) do
		local button = _G[name..index]
		local nextButton = _G[name..(index + 1)]
		local nextAnchor = "LEFT"
		local nextOffset = 0
		
		-- show/hide reagent trackers
		if buff.reagentName and not buff.disabled and TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			local icon = _G[name..index.."Icon"]
			button:Show()
			-- display spell or reagent icon
			if TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons") then
				icon:SetTexture(buff.spellIcon)
			else
				icon:SetTexture(buff.reagentIcon)
			end
			
			-- current number of reagents
			button:SetText(GetItemCount(buff.reagentName))
			
			if nextButton then
				nextAnchor = "RIGHT"
				nextOffset = 6
			end
			
			button:SetWidth(icon:GetWidth() + button:GetTextWidth())
			totalWidth = totalWidth + button:GetWidth()
			
			offset = offset + 1
			tracking = true
		else
			button:Hide()
		end
		
		-- fix offset to next reagent tracker
		if nextButton then
			nextButton:SetPoint("LEFT", button, nextAnchor, nextOffset, 0)
		end
	end
	
	-- show label if no tracking is enabled
	local none = _G[name.."None"]
	if tracking then
		none:Hide()
	else
		none:Show()
		totalWidth = none:GetWidth() + 8
	end
	
	-- adjust width so other plugins are properly offset
	TitanPanelReagentTrackerButton:SetWidth(totalWidth + ((offset - 1) * 8))
end


function TitanPanelReagentTrackerButton_GetTooltipText()
	local tooltipText = " "
	
	-- reagent info in tooltip
	for index, buff in ipairs(possessed) do
		local reagent = buff.reagentName
		if reagent and not buff.disabled and TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index) then
			tooltipText = format("%s\n%s\t%s", tooltipText, reagent, TitanUtils_GetHighlightText(GetItemCount(reagent)))
		end
	end
	
	if #tooltipText > 1 then
		return tooltipText
	else
		return " \nNo reagents tracked."
	end
end


function TitanPanelRightClickMenu_PrepareReagentTrackerMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_REAGENTTRACKER_ID].menuText)
	
	local info = {}
	
	-- add meny entry for each possessed spell
	for index, buff in ipairs(possessed) do
		local reagent = buff.reagentName
		if reagent then
			info.text = "Track "..reagent
			info.value = "TrackReagent"..index
			info.func = TitanPanelReagentTracker_ToggleVar
			info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "TrackReagent"..index)
			info.disabled = buff.disabled
			info.keepShownOnClick = 1
			UIDropDownMenu_AddButton(info)
		end
	end
	
	TitanPanelRightClickMenu_AddSpacer()
	
	info.text = "Show spell icons"
	info.value = "ShowSpellIcons"
	info.func = TitanPanelReagentTracker_ToggleVar
	info.checked = TitanGetVar(TITAN_REAGENTTRACKER_ID, "ShowSpellIcons")
	info.keepShownOnClick = 1
	UIDropDownMenu_AddButton(info)
	
	TitanPanelRightClickMenu_AddSpacer()
	
	TitanPanelRightClickMenu_AddCommand(L["TITAN_PANEL_MENU_HIDE"], TITAN_REAGENTTRACKER_ID, TITAN_PANEL_MENU_FUNC_HIDE)
end


function TitanPanelReagentTracker_ToggleVar(self)
	TitanToggleVar(TITAN_REAGENTTRACKER_ID, self.value)
	TitanPanelReagentTracker_UpdateButton()
end